%{
@Name:ai_oneshot.m
@Author: Smacq
@Copyright: Smacq
@Date: 2025/02/08
@Brief:
The Ai functions of USB-5000, continuous acquisition.
Oneshot limit 110 data point,acquisition Ai0 channel 100 remain 10 data point.
You can look at user manual know more about functions or Device info.

USB-5000 only connect USB execution result (Matlab version, similar concept as Python example):
MATLAB is 64-bit.
Dll load success.
Device_number: 1
     0
Device_Index: 0 Device_SN: 66666666 Device_Model: 3323
     0
     0
ErrorCode =
     0
...
     0
remain_buffer: 2
Ai0 100 data points
  arrange 1 to 8
    0.0019    0.0019    0.0016         0   -0.0013   -0.0022   -0.0034   -0.0034
...
  arrange 97 to 100
   -0.0031   -0.0038   -0.0028   -0.0031
remain_buffer: 10
     0
     0
%}

% Clear workspace and command window
clear;
clc;

% Check MATLAB bit is 32 or 64, load different lib file
if exist('matlab\private\util', 'dir') ~= 7
    dll_file_path = '.\lib\x64\USB5000.dll';  % If MATLAB is 64-bit, use this DLL path
    fprintf('MATLAB is 64-bit.\n');
else
    dll_file_path = '.\lib\x86\USB5000.dll';  % If MATLAB is 32-bit, use this DLL path
    fprintf('MATLAB is 32-bit.\n');
end

h_file_path = '.\lib\USB5000.h';

try
    loadlibrary(dll_file_path, h_file_path);
    if libisloaded('USB5000')
        fprintf('Dll load success.\n');
    end
catch
    fprintf('Dll load Fail please check matlab bit and lib file path.\n');
end

% Initialize the array for storing the acquired data.
% Usually the array size need >= acquisition data points size.
% Data type is single (float32 in Python, equivalent in Matlab), it is not modifiable.
% The data points saved to Ai_data_array arrange from opened Ai channel.
Ai_data_array = zeros(1, 2400, 'single');

% Define pointer to Ai_data_array type is singlePtr
Ai_data_array_ptr = libpointer('singlePtr', Ai_data_array);

% Device index starts from zero
DevIndex = int32(0);

% Ai voltage range:
% 10 = -10V to +10V and so on
% Voltage range param support 10/5
VoltageRange = single(10);

% Ai channel sample mode:
% 0=Continuous acquisition or 1=Oneshot, param support 0/1
SamplingMode = uint8(1);

% Acquisition amount of data:
% Each opened Ai channel will get this amount of data.
% Suggest value within sampling rate 10%~20%.
NumOfDataPoints = uint32(100);

% Configuration values for Ai channels:
% Device model channel number start from 0
ChannelNumber = uint8(0);

% Set channel ON/OFF
ChannelOn = uint8(1);
ChannelOff = uint8(0);

% Ai Sampling period, unit ns, steps 10ns:
% USB-5000 is asynchronous DAQ, single channel max rate is 1M=1000.
% Multichannel each channel share out samplerate, 1M/channel.
% sample rate and period:
% 100k = 10000
% 10k  = 100000
% 1k   = 1000000
% 100  = 10000000
% 10   = 100000000
% 1    = 1000000000
SamplingPeriod = uint32(1000000);

% Ai acquisition trig source:
% 0=SetUSB5AiSoftTrig(), 1=Ext_Trig_In, 2~5=Din_0~3, 
% 7=with DI, 8=with DO, 9~12=with AO~3, 255=GlobalSoftTrig
% param support 0/1/2/3/4/5/7/8/9/10/11/12/255
TriggerSource = uint8(0);

% Ai acquisition clock source:
% 0=SetUSB5AiSampleRate(), 1=Conv_In, 3=Di, 4=DO, 5~8=AO~3
% param support 0/1/3/4/5/6/7/8
ClockSource = uint8(0);

% Timeout unit ms:
% Suggest Timeout>(Num*Channel_count/Samplerate)*1000
% Avoid USB5GetAi() func timeout, if timeout return -7.
TimeoutValue = int32(200);

% AiOneShotPoints > Num, limit 110(set number need -1) data point
AiOneShotPoints = uint32(111);

% When acquisition amount is 0, just check FIFO return:
Zero = uint32(0);

% Find connected DAQ number
DeviceNumber = calllib('USB5000', 'FindUSB5DAQ');
disp(['Device_number: ', num2str(DeviceNumber)]);

% Open specific device and print return value.
ErrorCode = calllib('USB5000', 'USB5OpenDevice', DevIndex);
disp(ErrorCode);

% Get and print Device_SN
DeviceSerialNumber = calllib('USB5000', 'USB5ReadDevcieSN', DevIndex);
DeviceModel = calllib('USB5000', 'USB5ReadDevcieModel', DevIndex);
disp(['Device_Index: ', num2str(DevIndex), ' Device_SN: ', num2str(DeviceSerialNumber), ' Device_Model: ', num2str(DeviceModel)]);

% Turn OFF Ai channel 0-8 (default Ai channel 0-8 is ON)
for Channel = 0:7
    ErrorCode = calllib('USB5000', 'SetUSB5AiChanSel', DevIndex, Channel, ChannelOff);
    if ErrorCode ~= 0
        disp(ErrorCode);
    end
end

% Turn ON and set input voltage Range for Ai channel 0
for Channel = 0:0
    ErrorCode = calllib('USB5000', 'SetUSB5AiChanSel', DevIndex, Channel, ChannelOn);
    disp(ErrorCode);
    ErrorCode = calllib('USB5000', 'SetUSB5AiRange', DevIndex, Channel, VoltageRange);
    disp(ErrorCode);
end

% Don't need SetUSB5ClrAiTrigger(), set limit 110 data point acquisition
ErrorCode = calllib('USB5000', 'SetUSB5AiOneShotPoints', DevIndex, AiOneShotPoints)
disp(ErrorCode);

% Set Device Ai sample mode
ErrorCode = calllib('USB5000', 'SetUSB5AiSampleMode', DevIndex, SamplingMode);
disp(ErrorCode);

% Set Device sample rate
ErrorCode = calllib('USB5000', 'SetUSB5AiSampleRate', DevIndex, SamplingPeriod);
disp(ErrorCode);

% Set Device clock source
ErrorCode = calllib('USB5000', 'SetUSB5AiConvSource', DevIndex, ClockSource);
disp(ErrorCode);

% Clear AI FIFO
ErrorCode = calllib('USB5000', 'SetUSB5ClrAiFifo', DevIndex);
disp(ErrorCode);

% Set Device trig source
ErrorCode = calllib('USB5000', 'SetUSB5AiTrigSource', DevIndex, TriggerSource);
disp(ErrorCode);

% Enable Ai software trig acquisition
ErrorCode = calllib('USB5000', 'SetUSB5AiSoftTrig', DevIndex);
disp(ErrorCode);

% Continuous acquisition execution 10 times GetAiChans()
RemainingBuffer = calllib('USB5000', 'USB5GetAi', DevIndex, NumOfDataPoints, Ai_data_array_ptr, TimeoutValue);
% remain_buffer range 0~2000000
% USB5GetAi() return the remaining Device FIFO buffer value.
% FIFO buffer 100 data points save to Ai_data_array_pt.
% Channel_0 100 data points is Ai_data_array(1:100)
% If open channel 0~1, Channel_1 100 data points is Ai_data_array(100:200) and so on.
% If remain_buffer increase you should run USB5GetAi() more quick, let remain_buffer steady.
disp(['remain_buffer: ', num2str(RemainingBuffer)]);
disp(['Ai0 100 data points']);
Ai_data_array = Ai_data_array_ptr.value;
disp(Ai_data_array(1:100));

pause(3)
% Print and check FIFO remain 10 data point
RemainingBuffer = calllib('USB5000', 'USB5GetAi', DevIndex, Zero, Ai_data_array_ptr, TimeoutValue);
disp(['remain_buffer: ', num2str(RemainingBuffer)]);
% Add your code here can handle data

% Clear AI FIFO
ErrorCode = calllib('USB5000', 'SetUSB5ClrAiFifo', DevIndex);
disp(ErrorCode);

% Close Device
ErrorCode = calllib('USB5000', 'USB5CloseDevice', DevIndex);
disp(ErrorCode);

% Unload the dynamic link library
unloadlibrary('USB5000');