%{
@Name:ao_waveform_pc_infinite.m
@Author: Smacq
@Copyright: Smacq
@Date: 2025/02/08
@Brief:
The Ao functions of USB-5000, waveform output.
Ao channel 0 output -1V ~ 1V sine wave and -5V ~ +5V square wave.
Device AO Analog voltage range is -10V ~ +10V.
Ao channel is different for modul, Ao_4 or NONE.
You can look at user manual know more about functions or Device info.

USB-5100(Ao_4) only connect USB execution result (Matlab version, similar concept as Python example):
MATLAB is 64-bit.
Dll load success.
     0
...
     0
Ao0 outputting -1V and 1V sine wave, Please Enter to next wave.
     0
     0
     0
     0
Ao0 outputting -1V and 1V square wave, Please Enter to stop output.
    "0"    " AOChannel_0 is 0V"
     0
     0
     0
%}

% Clear workspace and command window
clear;
clc;

% Check MATLAB bit is 32 or 64, load different lib file
if exist('matlab\private\util', 'dir') ~= 7
    dll_file_path = '.\lib\x64\USB5000.dll';  % If MATLAB is 64-bit, use this DLL path
    fprintf('MATLAB is 64-bit.\n');
else
    dll_file_path = '.\lib\x86\USB5000.dll';  % If MATLAB is 32-bit, use this DLL path
    fprintf('MATLAB is 32-bit.\n');
end

h_file_path = '.\lib\USB5000.h';

try
    loadlibrary(dll_file_path, h_file_path);
    if libisloaded('USB5000')
        fprintf('Dll load success.\n');
    end
catch
    fprintf('Dll load Fail please check matlab bit and lib file path.\n');
end

% Init -1 ~ 1V sine wave to Ao_data_array
x = linspace(-2*pi, 2*pi, 1000);
Ao_data_array = single(sin(x));
% Initialize the array for storing the waveform data.
% Data type is single (float32 in Python, equivalent in Matlab), it is not modifiable.

% Define pointer to Ao_data_array type is singlePtr
Ao_data_array_ptr = libpointer('singlePtr', Ao_data_array);

% Device index starts from zero
DeviceIndex = int32(0);

% Ao channel, Ao channel 0 to Ao channel 3
AOChannel_0 = uint8(0);
AOChannel_1 = uint8(1);
AOChannel_2 = uint8(2);
AOChannel_3 = uint8(3);

% voltage value, range is -10 to +10
voltage_0 = single(0);

% Ao channel sample mode:
% 0=continuous non-loop output to computer buffer
% 1=FIFO output loop decide by SetUSB5AoCycle()
AoSampleMode = uint8(1);

% Ao Sampling period, unit ns, steps 10ns:
% USB-5000 is synchronous Ao, max rate is 100k = 10000.
% sample rate and period:
% 10k  = 100000
% 1k   = 1000000
% 100  = 10000000
% 10   = 100000000
% 1    = 1000000000
AoSampleRate = uint32(10000000);

% Ao acquisition trig source:
% 0=SetUSB5AoSoftTrig(), 1=Ext_Trig_In, 2~5=Din_0~3, 
% 6=with Ai, 7=with Di, 7=with Do, 9~12=with AO~3, 255=GlobalSoftTrig
% param support 0/1/2/3/4/5/6/7/8/9/10/11/12/255
AoTrigSource = uint8(0);

% Ao acquisition clock source:
% 0=SetUSB5AoSampleRate(), 1=Conv_In, 2=Ai, 3=Di, 5~8=AO~3
% param support 0/1/2/3/5/6/7/8
AoConvSource = uint8(0);

% Set waveform output cycle
% 0=infinite loop output
% 1=1 times, 2= 2 times and so on.
% param support 0~4294967295
AoCycle = uint32(0);

% Download to Ao FIFO data number
Num = uint32(length(Ao_data_array));

% Open specific device and print return value.
ErrorCode = calllib('USB5000', 'USB5OpenDevice', DeviceIndex);
disp(ErrorCode);

% Disable Ao software trig, clear flag
ErrorCode = calllib('USB5000', 'SetUSB5ClrAoTrigger', DeviceIndex, AOChannel_0);
disp(ErrorCode);

% Clear FIFO
ErrorCode = calllib('USB5000', 'SetUSB5ClrAoFifo', DeviceIndex, AOChannel_0);
disp(ErrorCode);

% Set Device Ao sample mode
ErrorCode = calllib('USB5000', 'SetUSB5AoSampleMode', DeviceIndex, AOChannel_0, AoSampleMode);
disp(ErrorCode);

% Set Device sample rate
ErrorCode = calllib('USB5000', 'SetUSB5AoSampleRate', DeviceIndex, AOChannel_0, AoSampleRate);
disp(ErrorCode);

% Set Device trig source
ErrorCode = calllib('USB5000', 'SetUSB5AoTrigSource', DeviceIndex, AOChannel_0, AoTrigSource);
disp(ErrorCode);

% Set Device clock source
ErrorCode = calllib('USB5000', 'SetUSB5AoConvSource', DeviceIndex, AOChannel_0, AoConvSource);
disp(ErrorCode);

% Set digital output cycle 2 times
ErrorCode = calllib('USB5000', 'SetUSB5AoCycle', DeviceIndex, AOChannel_0, AoCycle);
disp(ErrorCode);

% Download array data to Ao FIFO
ErrorCode = calllib('USB5000', 'SetUSB5AoDataFifo', DeviceIndex, AOChannel_0, Ao_data_array_ptr, Num);
disp(ErrorCode);

% Enable Ao software trig start Ao output wave
ErrorCode = calllib('USB5000', 'SetUSB5AoSoftTrig', DeviceIndex, AOChannel_0);
disp(ErrorCode);

% Pause and wait for user input to continue to next wave
input('Ao0 outputting -1V and 1V sine wave, Please Enter to next wave.');

% Disable Ao software trig, clear flag
ErrorCode = calllib('USB5000', 'SetUSB5ClrAoTrigger', DeviceIndex, AOChannel_0);
disp(ErrorCode);

% Clear FIFO
ErrorCode = calllib('USB5000', 'SetUSB5ClrAoFifo', DeviceIndex, AOChannel_0);
disp(ErrorCode);

% Init -1V and 1V square wave 200 data point to Ao_data_array
x = ones(100, 'single');
y = -ones(100, 'single');
Ao_data_array = [x; y];
% Define pointer to Ao_data_array type is singlePtr
Ao_data_array_ptr = libpointer('singlePtr', Ao_data_array);
% Download to Ao FIFO data number
Num = uint32(length(Ao_data_array));
% Download array data to Ao FIFO
ErrorCode = calllib('USB5000', 'SetUSB5AoDataFifo', DeviceIndex, AOChannel_0, Ao_data_array_ptr, Num);
disp(ErrorCode);

% Enable Ao software trig start Ao output wave
ErrorCode = calllib('USB5000', 'SetUSB5AoSoftTrig', DeviceIndex, AOChannel_0);
disp(ErrorCode);

% Pause and wait for user input to stop output
input('Ao0 outputting -1V and 1V square wave, Please Enter to stop output.');

% Set AOChannel_0 to 0V
ErrorCode = calllib('USB5000', 'SetUSB5AoImmediately', DeviceIndex, AOChannel_0, voltage_0);
disp([ErrorCode, " AOChannel_0 is 0V"]);

% Disable Ao software trig, clear flag
ErrorCode = calllib('USB5000', 'SetUSB5ClrAoTrigger', DeviceIndex, AOChannel_0);
disp(ErrorCode);

% Clear FIFO
ErrorCode = calllib('USB5000', 'SetUSB5ClrAoFifo', DeviceIndex, AOChannel_0);
disp(ErrorCode);

% Close Device and stop Ao wave output
ErrorCode = calllib('USB5000', 'USB5CloseDevice', DeviceIndex);
disp(ErrorCode);

% Unload the dynamic link library
unloadlibrary('USB5000');