%{
@Name:di_continuous.m
@Author: Smacq
@Copyright: Smacq
@Date: 2025/02/08
@Brief:
The Di functions of USB-5000, digit input.
Acquisition Di0 channel 100 data point of 10 times.
DI default suspended is 3.3V HIGH, 0~1.2V is LOW.
DI channel is different for modul, 4DI or 8DI.
You can look at user manual know more about functions or Device info.

USB-5000(DI_2) only connect USB execution result (Matlab version, similar concept as Python example):
MATLAB is 64-bit.
Dll load success.
     0
...
     0
remain_buffer: 2
Di_channel 100 data points
  arrange 1 to 17
    3   3   3   3   3   3   3   3   3   3   3   3   3   3  
...
  arrange 86 to 100
    3   3   3   3   3   3   3   3   3   3   3   3   3   3  
     0
     0
     0
%}

% Clear workspace and command window
clear;
clc;

% Check MATLAB bit is 32 or 64, load different lib file
if exist('matlab\private\util', 'dir') ~= 7
    dll_file_path = '.\lib\x64\USB5000.dll';  % If MATLAB is 64-bit, use this DLL path
    fprintf('MATLAB is 64-bit.\n');
else
    dll_file_path = '.\lib\x86\USB5000.dll';  % If MATLAB is 32-bit, use this DLL path
    fprintf('MATLAB is 32-bit.\n');
end

h_file_path = '.\lib\USB5000.h';

try
    loadlibrary(dll_file_path, h_file_path);
    if libisloaded('USB5000')
        fprintf('Dll load success.\n');
    end
catch
    fprintf('Dll load Fail please check matlab bit and lib file path.\n');
end

% Device index starts from zero
DeviceIndex = int32(0);

% Init Di_data_array
Di_data_array = zeros(100, 'uint8');
% Usually the array size need >= acquisition data points size.
% The data points saved to Di_data_array arrange from opened Di channel
% Data type is uint8, it is not modifiable.
% Di_channel_4:0~0xF(1111) Di3~DI0, Di_channel_8:0~0xFF(1111 1111) Di7~DI0.

% Define pointer to Di_data_array type is uint8Ptr
Di_data_array_ptr = libpointer('uint8Ptr', Di_data_array);

% Di channel sample mode:
% 0=Continuous acquisition or 1=Oneshot, param support 0/1
DiSampleMode = uint8(0);

% Acquisition amount of data:
% Each opened Di channel will get this amount of data.
% Suggest value within sampling rate 10%~20%.
Points = uint64(100);

% Di Sampling period, unit ns, steps 10ns:
% USB-5000 is synchronous DI, channel max rate is 10M=100.
% sample rate and period:
% 1M   = 1000
% 100k = 10000
% 10k  = 100000
% 1k   = 1000000
% 100  = 10000000
% 10   = 100000000
% 1    = 1000000000
DiSamplePeriod = uint32(1000000);

% Di acquisition trig source:
% 0=SetUSB5DiSoftTrig(), 1=Ext_Trig_In, 2~5=Din_0~3, 
% 6=with Di, 8=with DO, 9~12=with AO~3, 255=GlobalSoftTrig
% param support 0/1/2/3/4/5/6/8/9/11/12/255
DiTrigSource = uint8(0);

% Di acquisition clock source:
% 0=SetUSB5DiSampleRate(), 1=Conv_In, 2=Di, 4=DO, 5~8=AO~3
% param support 0/1/2/4/5/6/7/8
DiConvSource = uint8(0);

% Timeout unit ms:
% Suggest Timeout>(Num*Channel_count/Samplerate)*1000
% Avoid USB5GetDi() func timeout, if timeout return -7.
Timeout = int32(200);

% Open specific device and print return value.
ErrorCode = calllib('USB5000', 'USB5OpenDevice', DeviceIndex);
disp(ErrorCode);

% Set Device Di sample mode
ErrorCode = calllib('USB5000', 'SetUSB5DiSampleMode', DeviceIndex, DiSampleMode);
disp(ErrorCode);

% Set Device sample rate
ErrorCode = calllib('USB5000', 'SetUSB5DiSampleRate', DeviceIndex, DiSamplePeriod);
disp(ErrorCode);

% Set Device clock source
ErrorCode = calllib('USB5000', 'SetUSB5DiConvSource', DeviceIndex, DiConvSource);
disp(ErrorCode);

% Clear Di FIFO
ErrorCode = calllib('USB5000', 'SetUSB5ClrDiFifo', DeviceIndex);
disp(ErrorCode);

% Set Device trig source
ErrorCode = calllib('USB5000', 'SetUSB5DiTrigSource', DeviceIndex, DiTrigSource);
disp(ErrorCode);

% Enable Di software trig acquisition
ErrorCode = calllib('USB5000', 'SetUSB5DiSoftTrig', DeviceIndex);
disp(ErrorCode);

% Continuous acquisition execution 10 times GetDiChans()
for i = 1:10
    % USB5GetDi() return the remDining Device FIFO buffer value.
    % FIFO buffer 100 data points save to Di_data_array_pt.
    % Channel_0 100 data points is Di_data_array(1:100).
    % If remain_buffer increase you should run USB5GetDi() more quick, let remain_buffer steady.
    remain_buffer = calllib('USB5000', 'USB5GetDi', DeviceIndex, Points, Di_data_array_ptr, Timeout);
    fprintf('remain_buffer: %d\n', remain_buffer);
    fprintf('Di_channel 100 data points\n');
    Di_data_array = Di_data_array_ptr.Value;
    disp(Di_data_array(1:100));
    % Add your code here can handle data
end

% Disable Di software trig acquisition
ErrorCode = calllib('USB5000', 'SetUSB5ClrDiTrigger', DeviceIndex);
disp(ErrorCode);

% Clear Di FIFO
ErrorCode = calllib('USB5000', 'SetUSB5ClrDiFifo', DeviceIndex);
disp(ErrorCode);

% Close Device
ErrorCode = calllib('USB5000', 'USB5CloseDevice', DeviceIndex);
disp(ErrorCode);

% Unload the dynamic link library
unloadlibrary('USB5000');